
/******************************************************************************
* MODULE     : anfunc_test.cc
* DESCRIPTION: Test analytic functions
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "complex_ball.hpp"
#include "complex.hpp"
#include "linear_anfunc.hpp"
using namespace mml;

#define R          real
#define C          complex
#define Polynomial polynomial<C>
#define Series     series<C>
#define Anfunc     anfunc<C>
//#define Variant    dense_vector<naive_multiply>
#define Variant    dense_vector<linear<naive_multiply,nat_value<2> > >

int
main () {
  C I (R (0), R (1));
  Polynomial Z= seq (C (0), C (1));
  Polynomial P= seq (C (1), C (-2), C (3));
  Polynomial Q= seq (C (1), I, 2*I+1, 3*I-7);
  Anfunc f (1);
  Anfunc g (P);
  Anfunc h (Q);
  Anfunc z (Z);
  cout << "f\t\t= " << f << "\n";
  cout << "g\t\t= " << g << "\n";
  cout << "h\t\t= " << h << "\n";
  cout << "f+g\t\t= " << f+g << "\n";
  cout << "f-g\t\t= " << f-g << "\n";
  cout << "-g\t\t= " << -g << "\n";
  cout << "2 g\t\t= " << C (2) * g << "\n";
  cout << "g/2\t\t= " << g / C (2) << "\n";
  cout << "g`\t\t= " << integrate (g) << "\n";
  cout << "g^2\t\t= " << g*g << "\n";
  cout << "g^3\t\t= " << g*g*g << "\n";
  cout << "g:2\t\t= " << focus (g, C (2)) << "\n";
  cout << "g <* 1\t\t= " << shift_exp (g, 1) << "\n";
  cout << "g >* 1\t\t= " << shift_exp (g, -1) << "\n";
  cout << "1/(1-z)\t\t= " << 1 / (1-z) << "\n";
  cout << "sqrt(1+z)\t= " << sqrt (1+z) << "\n";
  cout << "|f,2|\t\t= " << tail_bound (f, 2) << "\n";
  cout << "|g,2|\t\t= " << tail_bound (g, 2) << "\n";
  cout << "|h,2|\t\t= " << tail_bound (h, 2) << "\n";
  cout << "|g`,2|\t\t= " << tail_bound (integrate (g), 2) << "\n";
  cout << "|g^2,2|\t\t= " << tail_bound (g*g, 2) << "\n";
  cout << "|g^3,4|\t\t= " << tail_bound (g*g*g, 4) << "\n";
  cout << "|g <* 1,2|\t= " << tail_bound (shift_exp (g, 1), 2) << "\n";
  cout << "|g >* 1,2|\t= " << tail_bound (shift_exp (g, -1), 2) << "\n";
  cout << "\n";

  vector<C,Variant> vc (C (0), 2); vc (0)= C (1);
  vector<Anfunc,Variant> vv (C (0), 2); vv (0)= z; vv (1)= g;
  vector<Anfunc,Variant> v= solve_lde_init (vv, vc);
  cout << "vv\t\t= " << vv << "\n";
  cout << "fun vv\t\t= " << as_anfunc (vv) << "\n";
  cout << "vec fun vv\t= " << as_vector (as_anfunc (vv)) << "\n";
  cout << "vc\t\t= " << vc << "\n";
  cout << "v\t\t= " << v  << "\n";

  matrix<Anfunc > m (Anfunc (0), 2, 2);
  m.ref (0, 0)= Anfunc (1);
  m.ref (1, 1)= Anfunc (z);
  matrix<Anfunc > ms= solve_lde (m);
  cout << "m\t\t= " << m << "\n";
  cout << "fun m\t\t= " << as_anfunc (m) << "\n";
  cout << "slde m\t\t= " << ms << "\n";
  cout << "\n";

  //Anfunc t= sqrt (1+z/10);
  //Anfunc t= 1/(1-z/10);
  //Anfunc t= cos (z);
  Anfunc t= asin (z/10);
  //Anfunc t= v[0];
  //Anfunc t= ms (0, 0);
  //Anfunc t= integrate (exp (-z*z));
  //Anfunc t= exp (exp (z));
  //Anfunc t= exp (exp (exp (z)));
  Anfunc u= sin (z);
  cout << "t\t\t= " << t << "\n";
  cout << "u\t\t= " << u << "\n";
  cout << "|t,0|\t\t= " << tail_bound (t, 0) << "\n";
  cout << "|t,2|\t\t= " << tail_bound (t, 2) << "\n";
  cout << "|t,4|\t\t= " << tail_bound (t, 4) << "\n";
  cout << "|t,6|\t\t= " << tail_bound (t, 6) << "\n";
  cout << "|t,8|\t\t= " << tail_bound (t, 8) << "\n";
  cout << "|t,100|\t\t= " << tail_bound (t, 100) << "\n";
  return 0;
}
