
/******************************************************************************
* MODULE     : numeric_extra.cc
* DESCRIPTION: This should be spread out elsewhere in the future
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_NUMERIC_EXTRA_HPP
#define __MML_NUMERIC_EXTRA_HPP
#include "math.h"
#include "complexify.hpp"
#include "complex_ball.hpp"
#include "fft_root.hpp"
#include "generic_extra.hpp"
namespace mml {

/******************************************************************************
* FFT for complexified numbers
******************************************************************************/

#define Fft_root fft_root<complexify<C> >

template<typename C> inline complexify<C>
invert (const complexify<C>& z) {
  return C(1) / z;
}

template<typename C>
class Fft_root {
MML_ALLOCATORS;
public:
  complexify<C> u;
  inline fft_root (): u (1) {}
  fft_root (nat n);
  inline fft_root (const complexify<C>& c): u (c) {}
  inline Fft_root& operator = (const Fft_root& x) {
    u= x.u; return *this; }
  inline bool operator == (const Fft_root& v) { return u == v.u; }
  inline bool operator != (const Fft_root& v) { return u != v.u; }
};

template<typename C>
fft_root<complexify<C> >::fft_root (nat n) {
  if (n == 1) u= C (1);
  else if (n == 2) u= C(-1);
  else {
    C angle= Pi (C) / (n>>1);
    u= gaussian (cos (angle), sin (angle));
  }
}

#undef Fft_root

/******************************************************************************
* Double constants and better printing of complexified doubles
******************************************************************************/

STMPL
struct constant_helper<double > {
  //static inline double nan ();
  //static inline double plus_infinity ();
  //static inline double minus_infinity ();
  //static inline double infinity ();
  //static inline double fuzz ();
  static inline double log2 () {
    return 0.6931471805599453094172321214581766L; }
  static inline double pi () {
    return 3.1415926535897932384626433832795029L; }
  //static inline double euler ();
};

typedef complexify<double> complex_double;

generic
as_generic (const complex_double& z) {
  if (abs (Im (z)) < abs (Re (z)) * 1.0e-4)
    return as_generic (Re (z));
  if (abs (Re (z)) < abs (Im (z)) * 1.0e-4)
    return gaussian (as_generic (0), as_generic (Im (z)));
  else return gaussian (as_generic (Re (z)), as_generic (Im (z)));
}

/******************************************************************************
* FFT for complex balls
******************************************************************************/

inline complex_ball
invert (const complex_ball& z) {
  return 1 / z;
}

template<>
fft_root<complex_ball >::fft_root (nat n) {
  if (n == 1) u= 1;
  else if (n == 2) u= -1;
  else {
    real_ball angle= Pi (real_ball) / (n>>1);
    u= gaussian (cos (angle), sin (angle));
  }
}

} // namespace mml
#endif // __MML_NUMERIC_EXTRA_HPP
