
/******************************************************************************
* MODULE     : Complex_ball_test.cc
* DESCRIPTION: Test complex balls
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "complex_ball.hpp"
//#include "floating.hpp"
//#include "complexify_interval.hpp"
using namespace mml;

#define Real_ball real_ball
#define Ball complex_ball
//#define Real_ball interval<floating<> >
//#define Ball complexify<Real_ball >

void
test_add_simple (int prec) {
  set_precision (prec);
  cout << "*** Additive arithmetic at precision " << prec << "\n\n";
  Ball x= gaussian (Real_ball (1), Real_ball (2));
  Ball y= 7;
  cout << "x\t= " << x << "\n";
  cout << "y\t= " << y << "\n";
  cout << "-x\t= " << (-x) << "\n";
  cout << "x+x\t= " << (x+x) << "\n";
  cout << "x+y\t= " << (x+y) << "\n";
  cout << "x-x\t= " << (x-x) << "\n";
  cout << "x-y\t= " << (x-y) << "\n";
  cout << "(x-x)+y\t= " << ((x-x)+y) << "\n";
  cout << "\n";
}

void
test_mul_simple (int prec) {
  set_precision (prec);
  cout << "*** Multiplicative arithmeric at precision " << prec << "\n\n";
  Ball x= Ball (2) + Ball (1);
  cout << "x\t= " << x << "\n";
  cout << "x * x\t= " << x*x << "\n";
  cout << "x * -x\t= " << x*(-x) << "\n";
  cout << "-x * x\t= " << (-x)*x << "\n";
  cout << "-x * -x\t= " << (-x)*(-x) << "\n";
  cout << "x / x\t= " << (x/x) << "\n";
  cout << "x / -x\t= " << (x/(-x)) << "\n";
  cout << "-x / x\t= " << ((-x)/x) << "\n";
  cout << "-x / -x\t= " << ((-x)/(-x)) << "\n";
  cout << "x*x / x\t= " << ((x*x)/x) << "\n";
  cout << "\n";
}

void
test_other_ops (int prec) {
  set_precision (prec);
  cout << "*** Other operations " << prec << "\n\n";
  Ball x= gaussian (Real_ball (2), Real_ball (3));
  Ball y= make_ball<Ball> (complex_ball_center(-1),
			   complex_ball_radius (5)) >> 1;
  cout << "x\t= " << x << "\n";
  cout << "y\t= " << y << "\n";
  cout << "x << 5\t= " << (x << 5) << "\n";
  cout << "x >> 3\t= " << (x >> 3) << "\n";
  cout << "y << 5\t= " << (y << 5) << "\n";
  cout << "abs(x)\t= " << (abs (x)) << "\n";
  cout << "abs(-x)\t= " << (abs (-x)) << "\n";
  cout << "abs(y)\t= " << (abs (y)) << "\n";
  cout << "arg(x)\t= " << (arg (x)) << "\n";
  cout << "arg(-x)\t= " << (arg (-x)) << "\n";
  cout << "arg(y)\t= " << (arg (y)) << "\n";
  cout << "\n";
}

void
test_mul_progression (int prec) {
  set_precision (prec);
  cout << "*** Multiplicative progression at precision " << prec << "\n\n";
  int i;
  Ball x= gaussian (Real_ball (1), Real_ball (2));
  Ball y= x;
  for (i=0; i<50; i++) {
    cout << i << "\t" << y << "\n";
    y *= x;
  }
  cout << "\n";
}

void
test_mul_overflow (int prec) {
  set_precision (prec);
  cout << "*** Multiplicative overflows at precision " << prec << "\n\n";
  int i;
  Ball x= gaussian (Real_ball (1), Real_ball (2));
  for (i=0; i<50; i++) {
    cout << i << "\t" << x << "\n";
    x *= x;
  }
  cout << "\n";
}

void
test_mul_underflow (int prec) {
  set_precision (prec);
  cout << "*** Multiplicative underflows at precision " << prec << "\n\n";
  int i;
  Ball x= gaussian (Real_ball (0.3));
  // Ball x= gaussian (Real_ball (0.3), Real_ball (0.1));
  // FIXME: bug in MPFR when replacing by uncommented line?
  for (i=0; i<50; i++) {
    cout << i << "\t" << x << "\n";
    x *= x;
  }
  cout << "\n";
}

void
test_special_funcs (int prec) {
  set_precision (prec);
  cout << "*** Special functions at precision " << prec << "\n\n";
  Ball x= 1;
  Ball y= gaussian (Real_ball (3e6));
  // Ball y= gaussian (Real_ball (3e20));
  // FIXME: bug in MPFR when replacing by uncommented line?
  Ball z= x - x/y;
  cout << "x\t\t= " << x << "\n";
  cout << "y\t\t= " << y << "\n";
  cout << "z\t\t= " << z << "\n";
  cout << "pi\t\t= " << Pi(Ball) << "\n";
  cout << "i\t\t= " << Imaginary(Ball) << "\n";
  cout << "NaN\t\t= " << Nan(Ball) << "\n";
  cout << "Infty\t\t= " << Infinity(Ball) << "\n";
  cout << "Fuzz\t\t= " << Fuzz(Ball) << "\n";
  cout << "sqrt (x)\t= " << sqrt (x) << "\n";
  cout << "sqrt (y)\t= " << sqrt (y) << "\n";
  cout << "exp (x)\t\t= " << exp (x) << "\n";
  cout << "exp (y)\t\t= " << exp (y) << "\n";
  cout << "exp (-x)\t= " << exp (-x) << "\n";
  cout << "exp (-y)\t= " << exp (-y) << "\n";
  cout << "log (x)\t\t= " << log (x) << "\n";
  cout << "log (y)\t\t= " << log (y) << "\n";
  cout << "cos (x)\t\t= " << cos (x) << "\n";
  cout << "cos (y)\t\t= " << cos (y) << "\n";
  cout << "sin (x)\t\t= " << sin (x) << "\n";
  cout << "sin (y)\t\t= " << sin (y) << "\n";
  cout << "tan (x)\t\t= " << tan (x) << "\n";
  cout << "tan (y)\t\t= " << tan (y) << "\n";
  cout << "arc sin (1/2)\t= " << asin (x>>1) << "\n";
  cout << "arc sin (z)\t= " << asin (z) << "\n";
  cout << "arc cos (1/2)\t= " << acos (x>>1) << "\n";
  cout << "arc tan (x)\t= " << atan (x) << "\n";
  cout << "arc tan (y)\t= " << atan (y) << "\n";
  cout << "arc tan (z)\t= " << atan (z) << "\n";
  cout << "\n";
}

void
test_all (int prec) {
  test_add_simple (prec);
  test_mul_simple (prec);
  test_other_ops (prec);
  test_mul_progression (prec);
  test_mul_overflow (prec);
  test_mul_underflow (prec);
  test_special_funcs (prec);
}

int
main () {
  test_all (30);
  test_all (100);
  test_all (200);

  cout << "infty\t\t= " << Infinity (Ball) << "\n";
  cout << "-infty\t\t= " << -Infinity (Ball) << "\n";
  cout << "infty+1\t\t= " << Infinity (Ball) + Ball (1) << "\n";
  cout << "1-infty\t\t= " << Ball (1) - Infinity (Ball) << "\n";
  cout << "2 infty\t\t= " << Ball (2) * Infinity (Ball) << "\n";
  cout << "1/infty\t\t= " << Ball (1) / Infinity (Ball) << "\n";
  cout << "infty^2\t\t= " << Infinity (Ball) * Infinity (Ball) << "\n";
  cout << "infty/infty\t= " << Infinity (Ball) / Infinity (Ball) << "\n";
  cout << "|infty|\t\t= " << abs (Infinity (Ball)) << "\n";
  cout << "center(infty)\t= " << center (Infinity (Ball)) << "\n";
  cout << "radius(infty)\t= " << radius (Infinity (Ball)) << "\n";
  cout << "infty << 3\t= " << (Infinity (Ball) << 3) << "\n";
  cout << "infty >> 3\t= " << (Infinity (Ball) >> 3) << "\n";
  return 0;
}
