
/******************************************************************************
* MODULE     : fft_bench.cpp
* DESCRIPTION: Bench FFT
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "tft.hpp"
using namespace mml;

#define K modular<FFTP>

nat
up_2 (nat i) {
  return 1 << (log_2 (i-1) + 1);
}

template<typename C> tft_tree<C>
make (nat n) {
  C* a= new C[n];
  for (nat i=0; i<n; i++)
    a[i]= i*i;
  tft_tree<C> t= tft_leaf<C> (log_2 (up_2 (n)));
  insert (t, 0, a, n);
  delete[] a;
  return t;
}

nat
encode (nat i, nat base) {
  if (i == 0) return 0;
  return base * encode (i>>1, base) + (i&1);
}

template<typename C> void
construct (tft_tree<C>& t, nat offset, nat n, nat dim, nat N, nat base) {
  if (dim == 0) {
    C a[1];
    a[0]= 1;
    insert (t, offset, a, 1);
  }
  else for (nat i=0; i<n; i++)
    construct (t, offset + (encode (i, base) << (dim-1)), n-i, dim-1, N, base);
}

template<typename C> tft_tree<C>
make (nat n, nat dim) {
  tft_tree<C> t= tft_leaf<C> (dim * log_2 (up_2 (n)));
  construct (t, 0, n, dim, up_2 (n), 1<<dim);
  simplify (t);
  return t;
}

void
bench_tft (nat n) {
  tft_tree<K> t= make<K> (n);
  nat count= 0;
  nat start= mml_time ();
  while (mml_time () < start + 200) {
    tft (t);
    count++;
  }
  double total= mml_time () - start;
  cout.width (10); cout << n << " : ";
  cout.width (18); cout << (total/(count*n)) << " ms ";
  cout.width (18); cout << (total/count) << " ms\n";
}

void
bench_tft (nat n, nat dim) {
  tft_tree<K> t= make<K> (n, dim);
  // bench_tft (size (t)); return;
  nat sz   = size (t);
  nat count= 0;
  nat start= mml_time ();
  while (mml_time () < start + 200) {
    tft (t);
    count++;
  }
  double total= mml_time () - start;
  cout.width (10); cout << sz << " : ";
  cout.width (18); cout << (total/(count*sz)) << " ms ";
  cout.width (18); cout << (total/count) << " ms\n";
}

void
count_tft (nat n, nat dim) {
  tft_tree<K> t= make<K> (n, dim);
  nat sz       = size (t);
  cross_count  = 0;
  tft (t);
  double total= cross_count;
  cout.width (10); cout << sz << " : ";
  cout.width (18); cout << (total/sz) << " average ";
  cout.width (18); cout << cross_count << " total\n";
}

int
main () {
  /*
  cross_count= 0;
  tft_tree<K> t= make<K> (19, 2);
  print (t);
  cout << "-----------------------------------\n";
  tft (t);
  cout << "-----------------------------------\n";
  print (t);
  cout << "-----------------------------------\n";
  cout << "count  = " << cross_count << "\n";
  cout << "size   = " << size (t) << "\n";
  cout << "average= " << ((double) cross_count) / size (t) << "\n";
  */

  /*
  cross_count= 0;
  tft_tree<K> t= make<K> (153, 1);
  print (t);
  cout << "-----------------------------------\n";
  tft (t);
  cout << "-----------------------------------\n";
  print (t);
  cout << "-----------------------------------\n";
  cout << "count  = " << cross_count << "\n";
  cout << "size   = " << size (t) << "\n";
  cout << "average= " << ((double) cross_count) / size (t) << "\n";
  */

  /*
  for (double i=16.0000001; i<1048576.1; i *= 1.0717734625362931)
    bench_tft ((nat) i);
  for (double i=8.0000001; i<1024.1; i *= 1.0717734625362931)
    bench_tft ((nat) i, 2);
  for (double i=166666; i<1200000.1; i *= 1.16)
    bench_tft ((nat) i);
  for (double i=100.0; i<200.0; i *= 1.0617734625362931)
    bench_tft ((nat) i, 3);
  for (double i=1000000.0; i<3000000.1; i *= 1.0717734625362931)
    bench_tft ((nat) i);
  for (double i=1410; i<2500.1; i *= 1.0717734625362931)
    bench_tft ((nat) i, 2);
  */

  /*
  for (double i=1000000.0; i<3000000.1; i *= 1.0717734625362931)
    count_tft ((nat) i, 1);
  for (double i=1410; i<2500.1; i *= 1.0717734625362931)
    count_tft ((nat) i, 2);
  for (double i=166666; i<1200000.1; i *= 1.16)
    count_tft ((nat) i, 1);
  for (double i=100.0; i<200.0; i *= 1.0617734625362931)
    count_tft ((nat) i, 3);
  */

  /*
  for (nat i=16; i<=(1<<21); i*=2) {
    cout << i << "\t";
    bench_tft (i, 1);
    cout << i+1 << "\t";
    bench_tft (i+1, 1);
  }

  cout << "-----------------------------------\n";
  for (nat i=4; i<=2048; i*=2) {
    cout << i << "\t";
    bench_tft (i, 2);
    cout << i+1 << "\t";
    bench_tft (i+1, 2);
  }

  cout << "-----------------------------------\n";
  for (nat i=2; i<=256; i*=2) {
    cout << i << "\t";
    bench_tft (i, 3);
    cout << i+1 << "\t";
    bench_tft (i+1, 3);
  }
  */

  cout << "-----------------------------------\n";
  for (nat i=2; i<=64; i*=2) {
    cout << i << "\t";
    bench_tft (i, 4);
    cout << i+1 << "\t";
    bench_tft (i+1, 4);
  }

  cout << "-----------------------------------\n";
  for (nat i=2; i<=32; i++) {
    cout << i << "\t";
    bench_tft (i, 5);
  }
  
  cout << "-----------------------------------\n";
  for (nat i=2; i<=16; i++) {
    cout << i << "\t";
    bench_tft (i, 6);
  }

  cout << "-----------------------------------\n";
  for (nat i=2; i<=16; i++) {
    cout << i << "\t";
    bench_tft (i, 7);
  }

  cout << "-----------------------------------\n";
  for (nat i=2; i<=8; i++) {
    cout << i << "\t";
    bench_tft (i, 8);
  }

  return 0;
}
