
/******************************************************************************
* MODULE     : mv_series_test.cc
* DESCRIPTION: Test multivariate series
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "rational.hpp"
#include "mv_series.hpp"
using namespace mml;

#define Series series<rational,multivariate<naive_multiply> >

int
main () {
  Series x (generic ("x"));
  Series y (generic ("y"));
  Series z (generic ("z"));
  Series f= exp (x+ y * exp (z));

  //cout << "x\t\t= " << x << "\n";
  //cout << "y\t\t= " << y << "\n";
  //cout << "x+y\t\t= " << x+y << "\n";
  //cout << "exp(x+y)\t= " << exp(x+y) << "\n";
  //cout << "x+y+exp(x+y)\t= " << (x+y)+exp(x+y) << "\n";

  cout << "x\t\t= " << expand (x, 5) << " + ...\n";
  cout << "y\t\t= " << expand (y, 5) << " + ...\n";
  cout << "exp(x)\t\t= " << expand (exp(x), 5) << " + ...\n";
  cout << "x+y\t\t= " << expand (x+y, 5) << " + ...\n";
  cout << "log(1+x+y)\t= " << expand (log (1+x+y), 5) << " + ...\n";
  cout << "exp(x+y)\t= " << expand (exp(x+y), 5) << " + ...\n";
  cout << "cos(x+y)\t= " << expand (cos(x+y), 5) << " + ...\n";
  cout << "sin(x+y)\t= " << expand (sin(x+y), 5) << " + ...\n";
  cout << "tan(x+y)\t= " << expand (tan(x+y), 5) << " + ...\n";
  cout << "exp(x+sin(y+z))\t= " << expand (exp(x+sin(y+z)), 5) << " + ...\n\n";

  cout << "f\t\t= " << expand (f, 5) << " + ...\n";
  cout << "df/dx\t\t= " << expand (derive (f, "x"), 5) << " + ...\n";
  cout << "df/dy\t\t= " << expand (derive (f, "y"), 5) << " + ...\n";
  cout << "df/dz\t\t= " << expand (derive (f, "z"), 5) << " + ...\n";
  cout << "I(f,x)\t\t= " << expand (integrate (f, "x"), 5) << " + ...\n";
  cout << "I(f,y)\t\t= " << expand (integrate (f, "y"), 5) << " + ...\n";
  cout << "I(f,z)\t\t= " << expand (integrate (f, "z"), 5) << " + ...\n";

  return 0;
}
