
/******************************************************************************
* MODULE     : series_test.cc
* DESCRIPTION: Test series
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "rational.hpp"
//#include "real_ball.hpp"
#include "linear_series.hpp"
using namespace mml;

#define C rational
//#define C real_ball
#define Series series<C>
#define Variant dense_vector<linear<naive_multiply,nat_value<2> > >

int
main () {
  Series z (1, 1);
  Series f (5, 3);
  Series g= 1 / (1 - z);
  Series h= seq (C (3), C (6), C (2));
  Series s= sin (z) / sin (z + z*z);

  // (void) set_series_formula_output (true);
  cout << "z\t= " << z << "\n";
  cout << "f\t= " << f << "\n";
  cout << "g\t= " << g << "\n";
  cout << "h\t= " << h << "\n";
  cout << "s\t= " << s << "\n";
  cout << "-f\t= " << -f << "\n";
  cout << "f+g\t= " << f+g << "\n";
  cout << "f-g\t= " << f-g << "\n";
  cout << "g-f\t= " << g-f << "\n";
  cout << "f*g\t= " << f*g << "\n";
  cout << "2*f\t= " << 2*f << "\n";
  cout << "f*2\t= " << f*2 << "\n";
  cout << "f/g\t= " << f/g << "\n";
  cout << "f/3\t= " << f/3 << "\n";
  cout << "f+=g\t= " << (f+=g) << "\n";
  cout << "f-=g\t= " << (f-=g) << "\n";
  cout << "f*=g\t= " << (f*=g) << "\n";
  cout << "f/=g\t= " << (f/=g) << "\n";
  cout << "f*=11\t= " << (f*=11) << "\n";
  cout << "f/=11\t= " << (f/=11) << "\n";
  cout << "f <* 2\t= " << shift_exp (f, 2) << "\n";
  cout << "f >* 2\t= " << shift_exp (f, -2) << "\n";
  cout << "g <* 2\t= " << shift_exp (g, 2) << "\n";
  cout << "g >* 2\t= " << shift_exp (g, -2) << "\n";
  cout << "g o z^3\t= " << dilate (g, 3) << "\n";
  cout << "g'\t= " << derive (g) << "\n";
  cout << "`g\t= " << integrate (g) << "\n";
  cout << "f=g\t= " << (f==g) << "\n";
  cout << "f!=g\t= " << (f!=g) << "\n";
  cout << "f<g\t= " << (f<g) << "\n";
  cout << "f<=g\t= " << (f<=g) << "\n";
  cout << "f>g\t= " << (f>g) << "\n";
  cout << "f>=g\t= " << (f>=g) << "\n\n";

  cout << "sqrt (z sin z)\t= " << sqrt (z * sin (z)) << "\n";
  cout << "1/(1+f)\t\t= " << 1/(1+f) << "\n";
  cout << "exp(f)\t\t= " << exp (f) << "\n";
  cout << "log(1+f)\t= " << log (1+f) << "\n";
  cout << "log_init(1+f,3)\t= " << log_init (1+f, C(3)) << "\n";
  cout << "exp(z*exp(f))\t= " << exp (z * exp (f)) << "\n";
  cout << "sqrt(1+f)\t= " << sqrt (1+f) << "\n";
  cout << "(1+f)^7\t\t= " << pow (1+f, C(7)) << "\n";
  cout << "cos_sin(f)\t= " << cos_sin (f) << "\n";
  cout << "sin(f)\t\t= " << sin (f) << "\n";
  cout << "cos(f)\t\t= " << cos (f) << "\n";
  cout << "tan(f)\t\t= " << tan (f) << "\n\n";
  //cout << "asin(z)\t\t= " << asin (z) << "\n";
  //cout << "acos(z)\t\t= " << acos (z) << "\n";
  //cout << "atan(z)\t\t= " << atan (z) << "\n\n";

  cout << "------------------------------------------------------\n\n";

  {
    series<integer> zero (0);
    series<integer> one (1);
    series<integer> z (1, 1);
    series<integer> a= one / (one - z);
    series<integer> b= a * a;

    cout << "a\t\t= " << a << "\n";
    cout << "b\t\t= " << b << "\n";
    cout << "b quo z\t\t= " << quo (b, z) << "\n";
    cout << "b quo (2 + z)\t= " << quo (b, 2 + z) << "\n";
    cout << "b quo 0\t\t= " << quo (b, zero) << "\n";
    cout << "b rem z\t\t= " << rem (b, z) << "\n";
    cout << "b rem (2 + z)\t= " << rem (b, 2 + z) << "\n";
    cout << "b rem 0\t\t= " << rem (b, zero) << "\n";
  }

  cout << "\n";
  cout << "------------------------------------------------------\n\n";

  vector<C,Variant> vc (C(0), 2); vc(0)= C(1);
  vector<Series,Variant> vv (C(0), 2); vv(0)= z; vv(1)= f;
  vector<Series,Variant> v= solve_lde_init (vv, vc);
  cout << "v\t= " << v << "\n";
  cout << "ser v\t= " << as_series (v) << "\n";

  matrix<Series > m (Series (0), 2, 2);
  m.ref (0, 0)= Series (1);
  m.ref (1, 1)= Series (z);
  cout << "m\t= " << m << "\n";
  cout << "ser m\t= " << as_series (m) << "\n";
  cout << "slde m\t= " << solve_lde (m) << "\n";

  return 0;
}
