
/******************************************************************************
* MODULE     : generic_extra.hpp
* DESCRIPTION: Extra functions for generic expressions
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __GENERIC_EXTRA_HPP
#define __GENERIC_EXTRA_HPP
#include "generic.hpp"
#include "generic_functions.hpp"
#include "generic_symbols.hpp"
namespace mml {

inline generic operator << (const generic& g1, const generic& g2) {
  return generic ("<<") (g1, g2); }
inline generic operator >> (const generic& g1, const generic& g2) {
  return generic (">>") (g1, g2); }
inline generic operator | (const generic& g1, const generic& g2) {
  return generic (MML_OR) (g1, g2); }
inline generic operator & (const generic& g1, const generic& g2) {
  return generic (MML_AND) (g1, g2); }
inline generic min (const generic& g1, const generic& g2) {
  return generic ("min") (g1, g2); }
inline generic max (const generic& g1, const generic& g2) {
  return generic ("max") (g1, g2); }
inline generic abs (const generic& g) {
  return generic ("abs") (g); }

template<> inline generic sqrt (const generic& g) {
  return generic (MML_SQRT) (g); }
template<> inline generic exp (const generic& g) {
  return generic (MML_EXP) (g); }
template<> inline generic log (const generic& g) {
  return generic (MML_LOG) (g); }
template<> inline generic cos (const generic& g) {
  return generic (MML_COS) (g); }
template<> inline generic sin (const generic& g) {
  return generic (MML_SIN) (g); }
template<> inline generic tan (const generic& g) {
  return generic (MML_TAN) (g); }
inline generic acos (const generic& g) { return generic (MML_ARCCOS) (g); }
inline generic asin (const generic& g) { return generic (MML_ARCSIN) (g); }
inline generic atan (const generic& g) { return generic (MML_ARCTAN) (g); }
inline generic trig (const generic& g) {
  return sqtuple (comma (cos (g), sin (g))); }

inline generic sqrt_init (const generic& g, const generic& c) {
  return generic ("sqrt_init") (g, c); }
inline generic log_init (const generic& g, const generic& c) {
  return generic ("log_init") (g, c); }
inline generic acos_init (const generic& g, const generic& c) {
  return generic ("acos_init") (g, c); }
inline generic asin_init (const generic& g, const generic& c) {
  return generic ("asin_init") (g, c); }
inline generic atan_init (const generic& g, const generic& c) {
  return generic ("atan_init") (g, c); }

inline generic arg (const generic& g) {
  return generic ("arg") (g); }
inline generic Re (const generic& g) {
  return generic ("Re") (g); }
inline generic Im (const generic& g) {
  return generic ("Im") (g); }
inline generic conj (const generic& g) {
  return generic ("conj") (g); }
inline generic gaussian (const generic& g1, const generic& g2) {
  return plus (g1, times (g2, "mathi")); }
inline generic polar (const generic& g1, const generic& g2) {
  return times (g1, exp (times (g2, "mathi"))); }

inline generic derive (const generic& g) {
  return generic (MML_DERIVE) (g); }
inline generic derive (const generic& g, const generic& v) {
  return generic (MML_DERIVE) (g, v); }
inline generic integrate (const generic& g) {
  return generic (MML_INTEGRATE) (g); }
inline generic integrate (const generic& g, const generic& v) {
  return generic (MML_INTEGRATE) (g, v); }
inline generic integrate_init (const generic& g, const generic& c) {
  return generic ("integrate_init") (g, c); }
inline generic access (const generic& g, const generic& x) {
  return generic (MML_ACCESS) (g, x); }
inline generic access (const generic& g, const generic& x, const generic& y) {
  return generic (MML_ACCESS) (g, x, y); }

template<typename C> struct constant_helper;
STMPL
struct constant_helper<generic> {
  static inline generic pi () { return MML_PI; }
  static inline generic log2 () { return log (generic (2)); }
  static inline generic euler () { return MML_EULER; }
  static inline generic catalan () { return MML_CATALAN; }
  static inline generic imaginary () { return MML_I; }
  static inline generic nan () { return MML_NAN; }
  static inline generic fuzz () { return MML_FUZZ; }
  static inline generic infinity () { return MML_INFINITY; }
  static inline generic plus_infinity () { return MML_INFINITY; }
  static inline generic minus_infinity () { return negate (infinity ()); }
};

} // namespace mml
#endif // __GENERIC_EXTRA_HPP
