
/******************************************************************************
* MODULE     : universe.hpp
* DESCRIPTION: Universes of rules how to compute with symbolic expressions
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __UNIVERSE_HPP
#define __UNIVERSE_HPP
#include "base.hpp"
namespace mml {

/******************************************************************************
* Universes
******************************************************************************/

class universe;

class universe_rep: public rep_struct {
public:
  bool scalar_commute;
  bool general_commute;
  bool scalar_expand;
  bool general_expand;
  bool scalar_power_add;
  bool general_power_add;
  bool scalar_power_multiply;
  bool general_power_multiply;
  bool scalar_power_expand;
  bool general_power_expand;

private:
  void normalize () {
    if (!general_commute) {
      general_power_add     = false;
      general_power_multiply= false;
    }
    if (!scalar_power_add || !scalar_power_multiply)
      scalar_power_expand= false;
    if (!general_power_add || !general_power_multiply)
      general_power_expand= false;
  }

public:
  inline universe_rep ():
    scalar_commute (true),
    general_commute (true),
    scalar_expand (true),
    general_expand (true),
    scalar_power_add (true),
    general_power_add (true),
    scalar_power_multiply (true),
    general_power_multiply (true),
    scalar_power_expand (true),
    general_power_expand (true) {}
  inline virtual ~universe_rep () {}

  friend class universe;
};

class universe {
INDIRECT_PROTO (universe, universe_rep)
public:
  inline universe (): rep (new universe_rep ()) {
    rep->normalize ();
  }
  inline universe (bool commute, bool expand): rep (new universe_rep ()) {
    rep->general_commute = commute;
    rep->general_expand  = expand;
    rep->normalize ();
  }
};
INDIRECT_IMPL (universe, universe_rep)

extern universe std_universe;
extern universe nc_universe;
extern universe exp_universe;
extern universe nc_exp_universe;

/******************************************************************************
* Rules container class
******************************************************************************/

#define TMPL template<const universe& U>
#define Rules rules<U>

TMPL
class rules {
public:
  static bool scalar_commute;
  static bool general_commute;
  static bool scalar_expand;
  static bool general_expand;
  static bool scalar_power_add;
  static bool general_power_add;
  static bool scalar_power_multiply;
  static bool general_power_multiply;
  static bool scalar_power_expand;
  static bool general_power_expand;
};

TMPL bool Rules::scalar_commute         = U->scalar_commute;
TMPL bool Rules::general_commute        = U->general_commute;
TMPL bool Rules::scalar_expand          = U->scalar_expand;
TMPL bool Rules::general_expand         = U->general_expand;
TMPL bool Rules::scalar_power_add       = U->scalar_power_expand;
TMPL bool Rules::general_power_add      = U->general_power_expand;
TMPL bool Rules::scalar_power_multiply  = U->scalar_power_expand;
TMPL bool Rules::general_power_multiply = U->general_power_expand;
TMPL bool Rules::scalar_power_expand    = U->scalar_power_expand;
TMPL bool Rules::general_power_expand   = U->general_power_expand;

#undef TMPL
#undef Rules

} // namespace mml
#endif // __UNIVERSE_HPP
