
/******************************************************************************
* MODULE     : generic.cpp
* DESCRIPTION: Generic expressions
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "generic_internal.hpp"
namespace mml {

/******************************************************************************
* Default output as lisp
******************************************************************************/

void
lisp_print (ostream& out, const generic& g) {
  if (is_atom (g)) out << as_string (g);
  else if (is_compound (g)) {
    nat i, n= N (g);
    out << "(";
    if (n>0) lisp_print (out, g[0]);
    for (i=1; i<n; i++) {
      out << " ";
      lisp_print (out, g[i]);
    }
    out << ")";
  }
  else lisp_print (out, flatten (g));
}

ostream&
operator << (ostream& out, const generic& g) {
  //lisp_print (out, g);
  pretty_print (out, flatten (g));
  return out;
}

/******************************************************************************
* Default input from lisp
******************************************************************************/

static generic
from_lisp (const string& s, nat& pos) {
  while (pos < N(s) && s[pos] == ' ') pos++;
  if (s[pos] == '(') {
    vector<generic> v;
    pos++;
    while (pos < N(s) && s[pos] != ')') {
      v << from_lisp (s, pos);
      while (pos < N(s) && s[pos] == ' ') pos++;
    }
    if (pos < N(s)) pos++;
    return generic_compound (v);
  }
  else {
    nat start= pos;
    while (pos < N(s) && s[pos] != ' ' && s[pos] != '(' && s[pos] != ')')
      pos++;
    return generic (s (start, pos));
  }
}

generic
from_lisp (const string& s) {
  nat pos= 0;
  return from_lisp (s, pos);
}

} // namespace mml
