
/******************************************************************************
* MODULE     : vector.hpp
* DESCRIPTION: Generic vectors
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_VECTOR_HPP
#define __MML_VECTOR_HPP
#include "iterator.hpp"
namespace mml {
class naive_multiply;
template<typename V> class dense_vector;
#define TMPL_DEF template<typename C, typename V=dense_vector<naive_multiply> >
#define TMPL template<typename C, typename V>
#define Vector vector<C,V>
TMPL class vector;

/******************************************************************************
* Abstract interface for generic vectors
******************************************************************************/

TMPL_DEF class vector_rep REP_STRUCT {};

TMPL // default argument declared in base.hpp
class vector {
INDIRECT_PROTO_2 (vector, vector_rep, C, V)
public:
  vector ();
  template<typename T> vector (const T& c);
  template<typename T> vector (const vector<T,V>& v);
  vector (const C& x);
  vector (const iterator<C>& it);
  C& operator () (nat i);
  C operator[] (nat i) const;
};
INDIRECT_IMPL_2 (vector, vector_rep, typename C, C, typename V, V)

TMPL nat  dim (const Vector& v);
TMPL bool is_scalar    (const Vector& v);
TMPL bool is_vectorial (const Vector& v);

/******************************************************************************
* Vector iteration
******************************************************************************/

TMPL_DEF
class vector_iterator_rep: public iterator_rep<C> {
  Vector v;  // the vector to traverse
  nat i;     // current position
protected:
  bool is_busy () { return i < dim (v); }
  void advance () { i++; }
  C current () { return v[i]; }
public:
  vector_iterator_rep (const Vector& v2): v (v2), i (0) {}
};

TMPL iterator<C>
iterate (const Vector& v) {
  return iterator<C> (new vector_iterator_rep<C,V> (v));
}

/******************************************************************************
* Hashing and generic output routines for vectors
******************************************************************************/

TMPL nat
hash (const Vector& v) {
  register nat i, h= 12345, n= dim (v);
  for (i=0; i<n; i++)
    h= (h<<1) ^ (h<<5) ^ (h>>27) ^ hash (v[i]);
  return h;
}

TMPL generic
as_generic (const Vector& v) {
  if (is_scalar (v)) return as_generic (v[0]);
  nat i, n= dim(v);
  generic args= comma ();
  for (i=0; i<n; i++)
    args= comma (args, as_generic (v[i]));
  return sqtuple (args);
}

PRINT_SUGAR(TMPL,Vector);

#undef TMPL_DEF
#undef TMPL
#undef Vector
} // namespace mml
#endif // __MML_VECTOR_HPP
